library(plyr)
library(ggplot2)
library(corrplot)
library(FactoMineR)
library(factoextra)
library(car)
library(reshape2)
library(grid)
library(stringr)
library(RColorBrewer)
library(scales)
library(multcomp)
library(lme4)
library(lsmeans)
library(gridExtra)
library(ggdendro)
library(dendextend)
library(dendextendRcpp)
library(survminer)


#*************************************************************************************************
# Reading in all the data
#*************************************************************************************************
img_to_barcode <- read.csv("../data/plant/SnapshotInfo1.csv",header = T,stringsAsFactors = F)
img_to_barcode <- img_to_barcode[img_to_barcode$tiles != "",]
colnames(img_to_barcode)[3] <- "Barcodes"

shapes <- read.table("shapes_april13.txt",header = F,stringsAsFactors = F,sep = " ")[,-21]
colnames(shapes) <- c("meta","area","hull_area","solidity","perimeter","width","height","cmx","cmy","hull_verticies","ex","ey","emajor","eminor","angle","eccen","circ","round","ar","fd")
shapes$id <- substring(as.character(sapply(shapes$meta,function(i) strsplit(i,"/")[[1]][3])),9)
shapes$imgname <- as.character(sapply(shapes$meta,function(i) strsplit(strsplit(i,"/")[[1]][4],"[.]")[[1]][1]))
shapes <- join(shapes,img_to_barcode[,c("id","Barcodes","timestamp")],by="id")
assoc <- read.csv("../data/plant/nov2017_assoc.csv",header=T,stringsAsFactors = F)
shapes <- join(shapes,assoc,by="Barcodes")
shapes$DAP <- as.numeric(sapply(shapes$timestamp, function(i){
  meta <- strsplit(i," ")[[1]][1]
  date <- sapply(meta,function(j){
    t <- strsplit(j,"-")[[1]]
    if(t[2] == 11){
      4+(as.numeric(t[3])-21)
    }else{
      13+as.numeric(t[3])
    }
  })
}))
shapes <- shapes[!(shapes$Barcodes %in% c("Cc00AA000000","Fm000ZZ051747")),]
shapes_list <- split(shapes,shapes$Barcodes)
adj_df <- data.frame("Barcodes"=names(shapes_list),"adj"=as.numeric(lapply(shapes_list,function(i) max(which(aggregate(data=i,area~DAP,FUN = "mean")[,2]<10))+3)))
shapes <- join(shapes,adj_df,by="Barcodes")
shapes[shapes$adj == -Inf,"adj"] <- max(shapes$DAP)
shapes <- shapes[shapes$Microbes != "Pheno_microbes",]
shapes$Nitrogen <- ordered(shapes$Nitrogen, levels=c(10,100),labels=c("10/10","100/100"))
shapes$Genotype <- ordered(shapes$Genotype, levels=c("BTx","Ch17"),labels=c("BTx623","China17"))

ch_shapes <- read.table("shapes_april13_ch.txt",header = F,stringsAsFactors = F,sep = " ")[,-22]
colnames(ch_shapes) <- c("meta_ch","area_ch","hull_area_ch","solidity_ch","perimeter_ch","width_ch","height_ch","cmx_ch","cmy_ch","hull_verticies_ch","ex_ch","ey_ch","emajor_ch","eminor_ch","angle_ch","eccen_ch","circ_ch","round_ch","ar_ch","fd_ch","det_ch")
ch_shapes$imgname <- as.character(sapply(ch_shapes$meta_ch,function(i) strsplit(strsplit(i,"/")[[1]][4],"[.]")[[1]][1]))
head(ch_shapes)
weight <- read.csv("../data/plant/shoot_weights.csv",header = T,stringsAsFactors = F)
area_convert <- 13.2*3.7/46856

s <- join(shapes,ch_shapes,by="imgname")
s <- s[s$id >= 207042,]
#s <- join(s,weight,by="Barcodes",type="right")
s$timestamp <- strptime(s$timestamp,format = "%Y-%m-%d %H:%M:%S")
s$hr <- as.POSIXlt(s$timestamp)$hour
s <- na.omit(s)
s$ZT <- sapply(as.numeric(s$hr),function(i)if(i < 7){17+i}else{i-7})

s$status <- sapply(s$hr,function(i) if(i %in% 7:21){"32°C Daytime"}else{"22°C Nighttime"})
#s <- s[s$Nitrogen == "100/100" & s$Microbes == "MgCl",]
s_end <- s[s$DAP == max(s$DAP,na.rm = T),]
s_end <- join(s_end,weight,by="Barcodes",type="right")
s_end <- na.omit(s_end)
head(s_end)


#*************************************************************************************************
# Plotting shapes, biomass and det
#*************************************************************************************************
wilcox.test(data=s,det_ch~status)

p <- ggplot(s,aes(ZT,det_ch))+
  geom_jitter(aes(color=status),width = 0.25)+
  scale_x_continuous(limits=c(0,19),breaks = seq(from=0,to=23,by=2))+
  #scale_x_continuous(limits=c(0,140),breaks = seq(from=0,to=225,by=25))+
  scale_color_manual(values=rev(c("darkorange","dodgerblue4")))+
  ylab("Deviance (D)")+
  xlab("ZT")+
  theme_light()+
  theme(plot.title = element_text(hjust = 0.5))+
  theme(axis.text = element_text(size = 14),
        axis.title= element_text(size = 18))+
  theme(strip.background=element_rect(fill="gray50"),
        strip.text.x=element_text(size=14,color="white"),
        strip.text.y=element_text(size=14,color="white"))
p
ggsave("ch_det_24hr.png",width=5.98,height=4,plot = p, dpi = 300)

area_ch_r <- round(cor(s_end$area_ch,s_end$shoot_weight)^2,3)
area_uc_r <- round(cor(s_end$area,s_end$shoot_weight)^2,3)

area_df <- data.frame("area"=s_end$area,"shoot_weight"=s_end$shoot_weight,"set"="Unstandardized",stringsAsFactors = F)
area_df <- rbind(area_df,data.frame("area"=s_end$area_ch,"shoot_weight"=s_end$shoot_weight,"set"="Standardized",stringsAsFactors = F))

library(ggthemes)
p <- ggplot(area_df,aes(area*area_convert,shoot_weight))+
  geom_point(aes(color=set))+
  geom_smooth(aes(color=set),method = "lm")+
  ylab("Biomass (g)")+
  xlab(~~Area~(cm^2))+
  scale_color_manual(values=colorblind_pal()(8)[c(4,8)])+
  scale_x_continuous(limits = c(8,130), breaks=c(25,50,75,100,125))+
  theme_light()+
  theme(axis.text = element_text(size = 14),
        axis.title= element_text(size = 18))+
  theme(strip.background=element_rect(fill="gray50"),
        strip.text.x=element_text(size=14,color="white"),
        strip.text.y=element_text(size=14,color="white"))
p
ggsave("ch_area_to_biomass.png", width=6.64,height=5.1,plot = p, dpi = 300)

p <- ggplot(s,aes(perimeter,perimeter_ch))+
  geom_point(aes(color=status))+
  geom_abline(intercept = 0,slope=1)+
  ggtitle("Perimeter")+
  xlab("Unstandardized")+
  ylab("Standardized")+
  scale_color_manual(values=rev(c("darkorange","dodgerblue4")))+
  #scale_color_gradientn(colors=c("red","red","red","gray10"))+
  theme_light()+
  theme(axis.text = element_text(size = 14),
        axis.title= element_text(size = 18),
        plot.title = element_text(size = 20,hjust = 0.5))+
  theme(strip.background=element_rect(fill="gray50"),
        strip.text.x=element_text(size=14,color="white"),
        strip.text.y=element_text(size=14,color="white"))
p
ggsave("ch_perimeter_before_after.png", width=6.64,height=5.1,plot = p, dpi = 600)

#*************************************************************************************************
# Associate shape before and after CH to day and night
#*************************************************************************************************
shapes <- c("area","hull_area","solidity","perimeter","width","height","cmx","cmy","hull_verticies","ex","ey","emajor","eminor","angle","eccen","circ","round","ar","fd")
H2 <- c()
for(e in shapes){
  e_ch <- paste(e,"_ch",sep = "")
  s$diff <- ((s[,e_ch])-(s[,e]))
  my_t <- t.test(data=s,diff~status)$p.value
  H2 <- rbind(H2,c(e,my_t))
}
H2 <- data.frame("shape"=as.character(H2[,1]),"pvalue"=as.numeric(H2[,2]),stringsAsFactors = F)
H2$shape <-  ordered(H2$shape,levels=H2$shape[order(H2$pvalue)])
H2$status <- sapply(H2$pvalue, function(i)if(i < 0.05){"S"}else{"NS"})


#*************************************************************************************************
# Forest plot for difference in shapes
#*************************************************************************************************
head(s)
shapes <- c("area","hull_area","solidity","perimeter","width","height","cmx","cmy","hull_verticies","ex","ey","emajor","eminor","angle","eccen","circ","round","ar","fd")
d <- c()
for(e in shapes){
  e_ch <- paste(e,"_ch",sep = "")
  temp <- data.frame("diff"=(s[,e]-s[,e_ch]),stringsAsFactors = F)
  temp$status <- s$status
  temp$diff <- (temp$diff - mean(temp$diff))/sd(temp$diff)
  d <- cbind(d,temp$diff)
}
d <- data.frame(d,stringsAsFactors = F)
colnames(d) <- shapes
d$status <- s$status
d$ZT <- s$ZT
d_melt <- reshape2::melt(d,id=c("status","ZT"))
d_mean <- data.frame(aggregate(data=d_melt,value~variable:status,FUN = "mean"),stringsAsFactors = F)
colnames(d_mean)[3] <- "mean"
d_sd <- data.frame(aggregate(data=d_melt,value~variable:status,FUN = "sd"),stringsAsFactors = F)
colnames(d_sd)[3] <- "sd"
d_join <- join(d_mean,d_sd,by=c("variable","status"))
d_join$variable <- ordered(d_join$variable,levels=H2$shape[order(H2$pvalue)])
limits <- aes(ymax = mean + sd, ymin=mean - sd)

p <- ggplot(data = d_join, aes(variable,mean,group=status))+
  #geom_hline(yintercept = 0,color="gray40")+
  #geom_hline(yintercept = c(-1.96,1.96),linetype="dashed",color="gray20")+
  geom_point(aes(color=status),position = position_dodge(0.5),size=3)+
  geom_errorbar(limits,position = position_dodge(0.5),width=0.25)+
  scale_y_continuous(limits=c(-5,5),oob = rescale_none)+
  scale_color_manual(values=rev(c("darkorange","dodgerblue4")))+
  ylab("Normalized Values")+
  theme_light()+
  theme(plot.title = element_text(hjust = 0.5))+
  theme(axis.text = element_text(size = 14),
        axis.title= element_text(size = 18))+
  theme(strip.background=element_rect(fill="gray50"),
        strip.text.x=element_text(size=14,color="white"),
        strip.text.y=element_text(size=14,color="white"))+
  theme(axis.text.x = element_text(angle = 45, hjust = 1),
        axis.title.x = element_blank())
p
ggsave("ch_effect_shapes_z.png", width=8.5,height=4.75,plot = p, dpi = 600)


#*************************************************************************************************
# Associate color before and after CH to day and night
#*************************************************************************************************
setwd("/media/jberry/Extra Drive 1/Danforth/Sorghum/color_homography/")

img_to_barcode <- read.csv("../data/plant/SnapshotInfo1.csv",header = T,stringsAsFactors = F)
img_to_barcode <- img_to_barcode[img_to_barcode$tiles != "",]
colnames(img_to_barcode)[3] <- "Barcodes"
assoc <- read.csv("../data/plant/nov2017_assoc.csv",header=T,stringsAsFactors = F)
weight <- read.csv("../data/plant/shoot_weights.csv",header = T,stringsAsFactors = F)

get_color <- function(file_name){
  color_data <- read.table(file_name,header = F,stringsAsFactors = F,sep = " ")[,-182]
  color_data$id <- as.character(sapply(color_data$V1,function(i) strsplit(strsplit(i,"/")[[1]][3],"snapshot")[[1]][2]))
  color_data$imgname <- as.character(sapply(color_data$V1,function(i) strsplit(strsplit(i,"/")[[1]][4],"[.]")[[1]][1]))
  
  color_data <- join(color_data,img_to_barcode[,c("id","Barcodes","timestamp")],by="id")
  color_data <- join(color_data,assoc,by="Barcodes")
  
  color_data$DAP <- as.numeric(sapply(color_data$timestamp, function(i){
    meta <- strsplit(i," ")[[1]][1]
    date <- sapply(meta,function(j){
      t <- strsplit(j,"-")[[1]]
      if(t[2] == 11){
        4+(as.numeric(t[3])-21)
      }else{
        13+as.numeric(t[3])
      }
    })
  }))
  color_data <- color_data[!(color_data$Barcodes %in% c("Cc00AA000000","Fm000ZZ051747")),]
  color_data <- color_data[color_data$Microbes != "Pheno_microbes",]
  color_data$Nitrogen <- ordered(color_data$Nitrogen, levels=c(10,100),labels=c("10/10","100/100"))
  color_data$Genotype <- ordered(color_data$Genotype, levels=c("BTx","Ch17"),labels=c("BTx623","China17"))
  color_data[,2:181] <- t(apply(color_data[,2:181],1,function(i){i/sum(i)}))*100
  color_data$timestamp <- strptime(color_data$timestamp,format = "%Y-%m-%d %H:%M:%S")
  color_data$hr <- as.POSIXlt(color_data$timestamp)$hour
  color_data$status <- sapply(color_data$hr,function(i) if(i %in% 7:21){"Daytime"}else{"Nighttime"})
  #color_data <- color_data[color_data$Barcodes %in% weight$Barcodes,]
  return(color_data)
}
color_uc <- get_color("color_april13.txt")
color_ch <- get_color("color_april13_ch.txt")

color_end <- color_uc[color_uc$DAP == max(color_uc$DAP),]
color_end <- color_end[color_end$Barcodes %in% weight$Barcodes,]
tail(color_end)


#*************************************************************************************************
# Color CAPS
#*************************************************************************************************
uc_sub <- na.omit(color_uc[color_uc$status == "Nighttime",])
uc_sub$set <- "Unstandardized"
ch_sub <- na.omit(color_ch[color_ch$status == "Nighttime",])
ch_sub$set <- "Standardized"

y <- uc_sub[,2:181]
x <- uc_sub[,c("Genotype","Nitrogen","status","hr","Microbes","DAP","set")]
cap_uc <- vegan::capscale(y~Nitrogen+Condition(DAP),x)
cap_uc <- data.frame(summary(cap_uc)$sites)
cap_uc$treatment <- x$Nitrogen
cap_uc$status <- x$status
cap_uc$microbes <- x$Microbes
cap_uc$set <- x$set
cap_uc$meta <- paste(x$set,x$Nitrogen,x$status)

y <- ch_sub[,2:181]
x <- ch_sub[,c("Genotype","Nitrogen","status","hr","Microbes","DAP","set")]
cap_ch <- vegan::capscale(y~Nitrogen+Condition(DAP),x)
cap_ch <- data.frame(summary(cap_ch)$sites)
cap_ch$treatment <- x$Nitrogen
cap_ch$status <- x$status
cap_ch$microbes <- x$Microbes
cap_ch$set <- x$set
cap_ch$meta <- paste(x$set,x$Nitrogen,x$status)

comb <- rbind(cap_uc,cap_ch)
comb$set <- ordered(comb$set, levels=c("Unstandardized","Standardized"))
comb$treatment <- ordered(comb$treatment,levels=c("100/100","10/10"),labels=c("Control N","Low N"))
head(comb)

p <- ggplot(comb,aes(eval(parse(text=colnames(comb)[1])),eval(parse(text=colnames(comb)[2]))))+
  facet_wrap(~set,scales = "free")+
  geom_point(aes(color=treatment),size=2,alpha=0.3)+
  stat_ellipse(aes(fill=treatment,color=treatment),geom = "polygon",alpha=0.25)+
  scale_color_manual(values = c(muted("orange",l=30,c=100),muted("purple",l=40,c=100)))+
  scale_fill_manual(values = c("orange","purple"))+
  #scale_y_continuous(limits = c(-4,2))+
  #scale_x_continuous(limits = c(-3,4))+
  xlab(colnames(test)[1])+
  ylab(colnames(test)[2])+
  theme_light()+
  theme(strip.background=element_rect(fill="gray50"),
        strip.text.x=element_text(size=14,color="white"),
        strip.text.y=element_text(size=14,color="white"))+
  theme(axis.title= element_text(size = 18))+
  theme(axis.text = element_text(size = 14))+
  theme(axis.ticks.length=unit(0.2,"cm"))
p
ggsave("ch_hue_caps_nighttime_only.png", width = 7.7,height = 3.98,plot = p, dpi = 300)


#*************************************************************************************************
# Histogram and KS
#*************************************************************************************************
hist_avg <- function(data,day){
  sub <- data[data$DAP==day,]
  test <- data.frame(do.call("rbind",lapply(split(sub,sub$status),function(t){
    data.frame(do.call("rbind",lapply(split(t,t$Geno),function(g){
      data.frame(do.call("rbind",lapply(split(g,g$Microbes),function(m){
        colMeans(m[,2:72],na.rm = T)
      }
      )))
    }
    )))
  })))
  return(test)
}

hist_sd <- function(data,day){
  sub <- data[data$DAP==day,]
  test <- data.frame(do.call("rbind",lapply(split(sub,sub$status),function(t){
    data.frame(do.call("rbind",lapply(split(t,t$Geno),function(g){
      data.frame(do.call("rbind",lapply(split(g,g$Microbes),function(m){
        apply(m[,2:72],2,function(i){sd(i,na.rm = T)})
      }
      )))
    }
    )))
  })))
  return(test)
}

plot_histo_response <- function(ret,ch_data,uc_data,day){
  if(!ret %in% c("data","plot")){
    cat("option ret must be either 'plot' or 'data' ")
    return("")
  }
  limits <- aes(ymax = value + 1.96*sd, ymin=value - 1.96*sd)
  
  dat <- ch_data
  ch_avg <- hist_avg(dat,day)
  ch_sd <- hist_sd(dat,day)
  ch_avg <- do.call("rbind",lapply(split(ch_avg,sapply(rownames(ch_avg),function(i){strsplit(i,"[.]")[[1]][1]})),function(j){colMeans(j,na.rm = T)}))
  ch_sd <- do.call("rbind",lapply(split(ch_sd,sapply(rownames(ch_sd),function(i){strsplit(i,"[.]")[[1]][1]})),function(j){colMeans(j,na.rm = T)}))
  ch_avg <- data.frame(melt(ch_avg))
  ch_avg$sd <- data.frame(melt(ch_sd))[,3]
  ch_avg <- ch_avg[as.numeric(str_sub(ch_avg$Var2,2,4)) <= 180,]
  ch_avg$bin <- (2*(as.numeric(str_sub(ch_avg$Var2,2,4))-2))
  ch_avg$Type <- "Corrected"
  
  dat <- uc_data
  uc_avg <- hist_avg(dat,day)
  uc_sd <- hist_sd(dat,day)
  uc_avg <- do.call("rbind",lapply(split(uc_avg,sapply(rownames(uc_avg),function(i){strsplit(i,"[.]")[[1]][1]})),function(j){colMeans(j,na.rm = T)}))
  uc_sd <- do.call("rbind",lapply(split(uc_sd,sapply(rownames(uc_sd),function(i){strsplit(i,"[.]")[[1]][1]})),function(j){colMeans(j,na.rm = T)}))
  uc_avg <- data.frame(melt(uc_avg))
  uc_avg$sd <- data.frame(melt(uc_sd))[,3]
  uc_avg <- uc_avg[as.numeric(str_sub(uc_avg$Var2,2,4)) <= 180,]
  uc_avg$bin <- (2*(as.numeric(str_sub(uc_avg$Var2,2,4))-2))
  uc_avg$Type <- "Uncorrected"
  
  color_comb <- rbind(ch_avg,uc_avg)
  color_comb$Type <- ordered(color_comb$Type, levels=c("Uncorrected","Corrected"),labels=c("Unstandardized","Standardized"))
  if(ret == "data"){
    return(color_comb)
  }else{
    p <- ggplot(data=color_comb,aes(bin,value))+
      #ggtitle(paste(as.character(genotype)," (Day ",day,")",sep="",collapse = ""))+
      facet_grid(Var1~Type)+
      geom_ribbon(limits,fill="gray80")+
      geom_line(aes(colour=bin),size=2)+
      scale_color_gradientn(colors=hue_pal(l=65)(180)[1:71])+
      scale_x_continuous(breaks=seq(from=0,to=142,by=45),limits = c(0,142))+
      scale_y_continuous(limits = c(-0.5,12),oob = rescale_none)+
      ylab("Percentage of Mask Explained")+
      xlab("Hue Channel")+
      theme_light()+
      theme(strip.background=element_rect(fill="gray50"),
            strip.text.x=element_text(size=14,color="white"),
            strip.text.y=element_text(size=14,color="white"))+
      theme(axis.title= element_text(size = 18))+
      theme(axis.text = element_text(size = 14))+
      theme(axis.ticks.length=unit(0.2,"cm"))+
      theme(legend.position='none')
    return(p)
  }
}
p <- plot_histo_response("plot",color_ch,color_uc,19)
p
ggsave("ch_effect_hue_day19_day_night_lowN_1sd.png", width=5.36,height=5.49,plot = p, dpi = 600)


p_dat <- plot_histo_response("data",color_ch,color_uc,19)
d_uc <- data.frame("bin"=seq(1,2*71,2),"value"=rescale(cumsum(p_dat$value[p_dat$Var1 == "Daytime" & p_dat$Type == "Unstandardized"])),"sample"="(1) Day Unstandardized",stringsAsFactors = F)
d_c <- data.frame("bin"=seq(1,2*71,2),"value"=rescale(cumsum(p_dat$value[p_dat$Var1 == "Daytime" & p_dat$Type == "Standardized"])),"sample"="(2) Day Standardized",stringsAsFactors = F)
n_uc <- data.frame("bin"=seq(1,2*71,2),"value"=rescale(cumsum(p_dat$value[p_dat$Var1 == "Nighttime" & p_dat$Type == "Unstandardized"])),"sample"="(3) Night Unstandardized",stringsAsFactors = F)
n_c <- data.frame("bin"=seq(1,2*71,2),"value"=rescale(cumsum(p_dat$value[p_dat$Var1 == "Nighttime" & p_dat$Type == "Standardized"])),"sample"="(4) Night Standardized",stringsAsFactors = F)
b <- rbind(d_uc,d_c,n_uc,n_c)

ks_c <- sqrt(-0.5*log(0.15/2))
test1 <- if(max(abs(n_uc$value-n_c$value)) > ks_c*sqrt((81+1784)/(81*1784))){"Significant"}else{"Not Significant"}
test2 <- if(max(abs(d_c$value-d_uc$value)) > ks_c*sqrt((81+1784)/(81*1784))){"Significant"}else{"Not Significant"}
test3 <- if(max(abs(n_c$value-d_c$value)) > ks_c*sqrt((81+1784)/(81*1784))){"Significant"}else{"Not Significant"}

p <- ggplot(b,aes(bin,value))+
  geom_line(aes(color=sample,linetype=sample),size=1.1)+
  scale_color_manual(values=c("darkorange","darkorange","dodgerblue4","dodgerblue4"))+
  scale_linetype_manual(values=c("dotted","solid","dotted","solid"))+
  ylab("Cumulative Distribution")+
  xlab("Hue Channel")+
  labs(subtitle = paste0(paste0("KS Test 3vs4 := ",test1),"\n",paste0("KS Test 1vs2 := ",test2),"\n",paste0("KS Test 2vs4 := ",test3)))+
  scale_x_continuous(breaks=seq(from=0,to=142,by=45),limits = c(0,142))+
  theme_light()+
  theme(strip.background=element_rect(fill="gray50"),
        strip.text.x=element_text(size=14,color="white"),
        strip.text.y=element_text(size=14,color="white"))+
  theme(axis.title= element_text(size = 18))+
  theme(axis.text = element_text(size = 14))+
  theme(axis.ticks.length=unit(0.2,"cm"))
p
ggsave("ch_effect_ks_day19_bothN.png", width=6.64,height=4.78,plot = p, dpi = 300)
